/**************************************************************************
 *  This file is part of QXmlEdit                                         *
 *  Copyright (C) 2011 by Luca Bellonda and individual contributors       *
 *    as indicated in the AUTHORS file                                    *
 *  lbellonda _at_ gmail.com                                              *
 *                                                                        *
 * This library is free software; you can redistribute it and/or          *
 * modify it under the terms of the GNU Library General Public            *
 * License as published by the Free Software Foundation; either           *
 * version 2 of the License, or (at your option) any later version.       *
 *                                                                        *
 * This library is distributed in the hope that it will be useful,        *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
 * Library General Public License for more details.                       *
 *                                                                        *
 * You should have received a copy of the GNU Library General Public      *
 * License along with this library; if not, write to the                  *
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
 * Boston, MA  02110-1301  USA                                            *
 **************************************************************************/

#include <QApplication>
#include "xmlEdit.h"
#include "xmleditglobals.h"
#include "config.h"
#include "mainwindow.h"
#include "element.h"
#include "utils.h"
#include "StartParams.h"
#include "services/loghandler.h"

#if !defined(QXMLEDIT_NOMAIN)

const char *APP_TITLE = QT_TR_NOOP("QXmlEdit");

#define QXMLEDIT_TRANSLATIONS_PREFIX "QXmlEdit_"

static bool decodeCommandLine(QApplication &app, StartParams *params);
static void startTanslator(QApplication *app);
static void initLogger();
static void todo();

static QTranslator qtLibTranslator;
static QTranslator qXmlEditTranslator;

static LogHandler logHandler;
static StartParams startParams ;

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);
    Q_INIT_RESOURCE(risorse);

    QCoreApplication::setOrganizationName(ORGANIZATION_NAME);
    QCoreApplication::setOrganizationDomain(ORGANIZATION_DOMAIN);
    QCoreApplication::setApplicationName(APPLICATION_NAME);
    QXmlEditGlobals::setAppTitle(APP_TITLE);

    qInstallMsgHandler(LogHandler::msgHandler);
    if(!Config::init()) {
        Utils::errorReadingUserSettings();
    }
    ApplicationData appData;
    appData.setLogger(&logHandler);
    initLogger();
    appData.init();
    startTanslator(&app);

    todo();

    Element::loadIcons();
    app.setWindowIcon(QIcon(":/tree/icon.png"));

    decodeCommandLine(app, &startParams);
    MainWindow mainWindow(false, &app, &appData);

    mainWindow.show();
    switch(startParams.type) {
    default:
    case StartParams::Nothing:
        mainWindow.triggersWelcomeDialog();
        break;
    case StartParams::OpenFile:
        mainWindow.loadFile(startParams.fileName);
        break;
    case StartParams::VisFile:
        mainWindow.loadVisFile(startParams.fileName);
        break;
    }
    int result = app.exec();
    appData.end();
    if(!Config::end()) {
        Utils::errorSavingUserSettings();
    }
    return result;
}

static void initLogger()
{
    bool isEnabled = Config::getBool(Config::KEY_LOGS_ENABLED, false);
    int logLevel = Config::getInt(Config::KEY_LOGS_LEVEL, FrwLogger::INFO);
    logHandler.setLevel((FrwLogger::ELogLevel)logLevel);
    logHandler.setEnabled(isEnabled);
}

static bool decodeCommandLine(QApplication &app, StartParams *params)
{
    QStringList args = app.arguments();
    if(args.size() > 1) {
        QString arg1 = args.at(1);

        if(arg1 == QString("-vis")) {
            if(args.size() > 2) {
                params->fileName = args.at(2);
                if(!params->fileName.isEmpty()) {
                    params->type = StartParams::VisFile ;
                    return true;
                } else {
                    params->parametersError = true ;
                }
            }
        } else {
            params->fileName = args.at(1);
            if(!params->fileName.isEmpty()) {
                params->type = StartParams::OpenFile ;
                return true;
            } else {
                params->parametersError = true ;
            }
        }
    }
    return false;
}

void msgBoxDebug(const QString message)
{
    QMessageBox msgBox;
    msgBox.setText(message);
    msgBox.exec();
}

/** \brief: insert here all the fundamentals todo using Utils::TODO_THIS_RELEASE().
  */
void todo()
{
    Utils::TODO_THIS_RELEASE("Attenzione alla command line se apro piu' finestre.");
}


static void startTanslator(QApplication *app)
{
    if(qtLibTranslator.load("qt_" + QLocale::system().name(),
                            QLibraryInfo::location(QLibraryInfo::TranslationsPath))) {
        app->installTranslator(&qtLibTranslator);
    }
    if(qXmlEditTranslator.load(QXMLEDIT_TRANSLATIONS_PREFIX + QLocale::system().name(), ApplicationData::getResourceDir() + QString("/") + QString(STRINGIZE(TRANLASTION_DIR)))) {
        app->installTranslator(&qXmlEditTranslator);
    }
}

#endif //QXMLEDIT_NOMAIN
