/**************************************************************************
 *  This file is part of QXmlEdit                                         *
 *  Copyright (C) 2013 by Luca Bellonda and individual contributors       *
 *    as indicated in the AUTHORS file                                    *
 *  lbellonda _at_ gmail.com                                              *
 *                                                                        *
 * This library is free software; you can redistribute it and/or          *
 * modify it under the terms of the GNU Library General Public            *
 * License as published by the Free Software Foundation; either           *
 * version 2 of the License, or (at your option) any later version.       *
 *                                                                        *
 * This library is distributed in the hope that it will be useful,        *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
 * Library General Public License for more details.                       *
 *                                                                        *
 * You should have received a copy of the GNU Library General Public      *
 * License along with this library; if not, write to the                  *
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
 * Boston, MA  02110-1301  USA                                            *
 **************************************************************************/


#include "testxsdnavigation.h"
#include "app.h"
#include "xsdeditor/navigationtree.h"
#include <QTest>

#include "xsdeditor/widgets/xsdgenericeditor.h"
#include "ui_xsdwindow.h"

#define  FILE_BASE_XSD  "../test/data/xsd/navigation.xsd"

//---------------------------

bool TestXSDWindow::testCopyElementActionExecute(XSchemaObject *object)
{
    return copyElementActionExecute(object);
}

//---------------------------


NavigationTree *TestXSDWindow::navigation()
{
    return ui->navigation;
}


TestXSDNavigation::TestXSDNavigation()
{
    xsdEditor = NULL ;
}

TestXSDNavigation::~TestXSDNavigation()
{
}

//---------------------------

bool TestXSDNavigation::checkTopNavigation(QTreeWidgetItem *topItem)
{
    if( topItem->childCount()!=0) {
        return false;
    }
    return true;
}

bool TestXSDNavigation::checkTopElements(QTreeWidgetItem *topItem)
{
    if( topItem->childCount()!=1) {
        return false;
    }
    QTreeWidgetItem *item = topItem->child(0);
    if(item->text(0)!= "e1") {
        return false;
    }
    return true;
}

bool TestXSDNavigation::checkTopTypes(QTreeWidgetItem *topItem)
{
    if( topItem->childCount()!=2) {
        return false;
    }
    QTreeWidgetItem *item = topItem->child(0);
    if(item->text(0)!= "Model") {
        return false;
    }
    item = topItem->child(1);
    if(item->text(0)!= "Plane") {
        return false;
    }
    return true;
}

bool TestXSDNavigation::checkTopAttributes(QTreeWidgetItem *topItem)
{
    if( topItem->childCount()!=1) {
        return false;
    }
    QTreeWidgetItem *item = topItem->child(0);
    if(item->text(0)!= "a1") {
        return false;
    }
    return true;
}
bool TestXSDNavigation::checkTopReferences(QTreeWidgetItem *topItem)
{
    if( topItem->childCount()!=1) {
        return false;
    }
    QTreeWidgetItem *item = topItem->child(0);
    if(item->text(0)!= "r1") {
        QVariant variant = item->data(0, Qt::UserRole);
        XSchemaObject*object = (XSchemaObject*) variant.value<void *>();
        if( NULL != object ) {
            switch( object->getType() ) {
            case SchemaTypeRedefine:
            case SchemaTypeImport:
            case SchemaTypeInclude:
                return true;
            default:
                return false;
            }
        }
        return false;
    }
    return true;
}
bool TestXSDNavigation::checkInnerElements(QTreeWidgetItem *topItem)
{
    if( topItem->childCount()!=3) {
        return false;
    }
    QTreeWidgetItem *item = topItem->child(0);
    if(item->text(0)!= "e2") {
        return false;
    }
    item = topItem->child(1);
    if(item->text(0)!= "e3") {
        return false;
    }
    item = topItem->child(2);
    if(item->text(0)!= "e4") {
        return false;
    }
    return true;
}

//---------------------------

bool TestXSDNavigation::openXsdViewer(App *app)
{
    xsdEditor = new TestXSDWindow(app->mainWindow()) ;
    QString xmlAsString = app->mainWindow()->getRegola()->getAsText();
    xsdEditor->show();
    xsdEditor->setFileName("test");
    xsdEditor->setTitle("test");
    xsdEditor->loadString(xmlAsString);
    QTest::qWait(200);
    if( xsdEditor->isInError() ) {
        return error("data not loaded");
    }
    return true;
}

bool TestXSDNavigation::checkNavigation()
{
    NavigationTree *navigationTree = xsdEditor->navigation();
    // get top level elements and check them
    //int	topLevelItemsCount = navigationTree->topLevelItemCount();
    QTreeWidgetItem *topNavigation = navigationTree->topLevelItem(0);
    if(!checkTopNavigation(topNavigation)) {
        return error("top navigation");
    }
    QTreeWidgetItem *topElements = navigationTree->topLevelItem(2);
    if(!checkTopElements(topElements)) {
        return error("top elements");
    }
    QTreeWidgetItem *topTypes = navigationTree->topLevelItem(4);
    if(!checkTopTypes(topTypes)) {
        return error("top types");
    }
    QTreeWidgetItem *topAttributes = navigationTree->topLevelItem(6);
    if(!checkTopAttributes(topAttributes)) {
        return error("top attributes");
    }
    QTreeWidgetItem *topReferences = navigationTree->topLevelItem(8);
    if(!checkTopReferences(topReferences)) {
        return error("top references");
    }
    QTreeWidgetItem *topInnerElements = navigationTree->topLevelItem(10);
    if(!checkInnerElements(topInnerElements)) {
        return error("inner elements");
    }

    return true;
}

//---------------------------


bool TestXSDNavigation::testNavigation()
{
    _testName = "testNavigation";
    App app;
    if(!app.init() ) {
        return error("init");
    }
    if(!app.mainWindow()->loadFile(QString(FILE_BASE_XSD))) {
        return error("load file");
    }
    if( !openXsdViewer(&app) ) {
        return false;
    }
    if(! checkNavigation() ) {
        return false;
    }
    return true ;
}

//---------------------------
